<?php
// $Id: pifr.admin.inc,v 1.1 2008/12/02 06:45:23 boombatower Exp $
/**
 * @file
 * Automatically review patches attached to issues.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Display a list of servers.
 *
 * @return string HTML output.
 */
function pifr_admin_server_list() {
  $servers = pifr_server_get_all();

  $header = array(t('Server id'), t('URL'), t('Path'), t('Type'), t('Operations'));
  $rows = array();
  foreach ($servers as $server) {
    $row = array();
    $row[] = $server->server_id;
    $row[] = $server->url;
    $row[] = $server->path;
    $row[] = pifr_server_role($server->type);
    $row[] = pifr_admin_render_operations(pifr_admin_server_operations($server));

    $rows[] = $row;
  }

  if ($rows) {
    return theme('table', $header, $rows);
  }
  return '<p>' . t('No servers') . '</p>';
}

/**
 * Get an array containing the server operation links.
 *
 * @param $server Server object.
 * @return array List of links.
 */
function pifr_admin_server_operations($server) {
  $links = array();
  $links['server_edit'] = array(
    'title' => t('edit'),
    'href' => "admin/settings/pifr/servers/edit/$server->server_id"
  );
  $links['server_delete'] = array(
    'title' => t('delete'),
    'href' => "admin/settings/pifr/servers/delete/$server->server_id"
  );
  return $links;
}

/**
 * Render a set of operations as links.
 *
 * @param array $operations Operations to render.
 * @return string HTML output.
 */
function pifr_admin_render_operations(array $operations) {
  $out = '';
  foreach ($operations as $operation) {
    $out .= l($operation['title'], $operation['href']) . ' | ';
  }
  return substr($out, 0, strlen($out) - 3); // Remove last task pipe.
}

/**
 * Server information form.
 */
function pifr_admin_server_form(&$form_state, $server_id = NULL) {
  $form = array();

  if ($server_id) {
    $server = pifr_server_get($server_id);
  }

  $form['#redirect'] = 'admin/settings/pifr/servers';
  $form['server'] = array(
    '#type' => 'fieldset',
    '#title' => t('Server')
  );
  $form['server']['server_id'] = array(
    '#type' => 'value',
    '#value' => $server_id
  );
  $form['server']['server_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Server key'),
    '#default_value' => (isset($server) ? $server->server_key : '')
  );
  $form['server']['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('The base URL of the server.'),
    '#default_value' => (isset($server) ? $server->url : '')
  );
  $form['server']['path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path'),
    '#description' => t('The path to the files directory.'),
    '#default_value' => (isset($server) ? $server->path : '')
  );
  $form['server']['type'] = array(
    '#type' => 'select',
    '#title' => t('Type'),
    '#description' => t('The type of server this record is to represent. For multiple types please make multiple server entries.'),
    '#options' => array(
      PIFR_SERVER_PROJECT,
      PIFR_SERVER_TEST_MASTER,
      PIFR_SERVER_SLAVE
    ),
    '#default_value' => (isset($server) ? $server->type : '')
  );
  $form['server']['type']['#options'] = drupal_map_assoc($form['server']['type']['#options'], 'pifr_server_role');

  $form['op'] = array(
    '#type' => 'submit',
    '#value' => t('Save')
  );

  return $form;
}

/**
 * Save server data.
 */
function pifr_admin_server_form_submit($form, &$form_state) {
  if ($form_state['values']['server_id']) {
    // Existing server, update record.
    db_query("UPDATE {pifr_server}
              SET server_key = '%s',
              url = '%s',
              path = '%s',
              type = %d
              WHERE server_id = %d",
              $form_state['values']['server_key'], $form_state['values']['url'],
              $form_state['values']['path'], $form_state['values']['type'], $form_state['values']['server_id']);
  }
  else {
    // New server, insert record.
    db_query("INSERT INTO {pifr_server} (server_key, url, path, type)
              VALUES ('%s', '%s', '%s', %d)",
              $form_state['values']['server_key'], $form_state['values']['url'],
              $form_state['values']['path'], $form_state['values']['type']);
  }
  drupal_set_message(t('Changes saved successfully.'));
}

/**
 * Confirm removal of server form.
 */
function pifr_admin_server_delete_confirm_form(&$form_state, $server_id) {
  $form = array();
  $form['server']['server_id'] = array(
    '#type' => 'value',
    '#value' => $server_id
  );

  return confirm_form($form,
                      t('Are you sure you want to delete server #@server_id', array('@server_id' => $server_id)),
                      'admin/settings/pifr/servers');
}

/**
 * Remove server.
 */
function pifr_admin_server_delete_confirm_form_submit($form, &$form_state) {
  db_query('DELETE FROM {pifr_server}
            WHERE server_id = %d', $form_state['values']['server_id']);

  $form_state['redirect'] = 'admin/settings/pifr/servers';

  drupal_set_message(t('Server removed.'));
}
