<?php
// $Id: pifr.pages.inc,v 1.48 2008/12/13 18:42:16 boombatower Exp $
/**
 * @file
 * Automatically review patches attached to issues.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Main landing page.
 *
 * @return string HTML output.
 */
function pifr_view() {
  $output = '';

  // List ways to view.
  $header = array(t('View by'), t('Pattern'));
  $rows = array();
  $rows[] = array(t('Node'), '<code>@base_urlpifr/node/<b>[project]</b>/<b>[node]</b></code>');
  $rows[] = array(t('File'), '<code>@base_urlpifr/file/<b>[project]</b>/<b>[file]</b></code>');

  $output .= t('<p>The current status or result of patch testing can be viewed in the following ways.</p>' .
               theme('table', $header, $rows), array('@base_url' => url('', array('absolute' => TRUE))));

  // List project servers.
  $servers = pifr_server_get_type(PIFR_SERVER_PROJECT);
  $projects = array();
  foreach ($servers as $server) {
    $url = parse_url($server->url);
    $projects[] = $url['host'] . ' (' . l(t('feed'), 'pifr/rss/' . $url['host']) . ')';
  }

  $output .= t('<p><b>Projects</b>!projects</p>', array('!projects' => theme('item_list', $projects)));

  return $output;
}

/**
 * View event log. If file ID is specified then return events for the specified
 * file only.
 *
 * @param integer $file_id File ID.
 * @return string HTML output.
 */
function pifr_log_view($file_id = NULL) {
  if ($file_id) {
    $events = pifr_log_get($file_id);
  }
  else {
    $events = pifr_log_get_all();
  }

  if ($file_id) {
    $header = array(t('Timestamp'), t('Message'));
  }
  else {
    $header = array(t('Timestamp'), t('File'), t('Message'));
  }
  $rows = array();

  foreach ($events as $event) {
    $row = array();
    $row[] = format_date($event->timestamp, 'small');
    if (!$file_id) {
      $file = pifr_file_get($event->file_id);
      $row[] = l($file->filename, 'pifr/file/' . $file->server_id . '/' . $file->file_id);
    }
    $row[] = $event->message;

    $rows[] = $row;
  }

  if ($rows) {
    return theme('table', $header, $rows) . theme('pager', NULL, ($file_id ? 10 : 50), 0);
  }
  return '<p><em>' . t('No events to display.') . '</em></p>';
}

/**
 * Search form.
 */
function pifr_search_form(&$form_state) {
  $form = array();

  $form['query'] = array(
    '#type' => 'textfield',
    '#title' => t('Query'),
    '#size' => 24,
    '#autocomplete_path' => 'pifr/search/autocomplete'
  );
  $form['op'] = array(
    '#type' => 'submit',
    '#value' => t('Search')
  );

  return $form;
}

/**
 * Perform a search query and redirect to the appropriate page.
 */
function pifr_search_form_submit($form, &$form_state) {
  $query = $form_state['values']['query'];
  if (is_numeric($query)) {
    // Assume looking for NID.
    $server_id = db_result(db_query('SELECT server_id
                                     FROM {pifr_file}
                                     WHERE nid = %d', $query));
    if ($server_id) {
      $form_state['redirect'] = "pifr/node/$server_id/$query";
      return;
    }
  }
  else {
    // Assume looking for filename.
    $server_id = db_result(db_query("SELECT server_id
                                     FROM {pifr_file}
                                     WHERE filename = '%s'", $query));
    if ($server_id) {
      $form_state['redirect'] = "pifr/file/$server_id/$query";
      return;
    }
  }
  drupal_set_message(t('Search returned no results.'), 'error');
}

/**
 * Get suggestions based on the search term.
 *
 * @param string $search Search query.
 */
function pifr_search_autocomplete($search) {
  if (is_numeric($search)) {
    // Assume looking for NID.
    $result = db_query("SELECT nid
                        FROM {pifr_file}
                        WHERE nid LIKE '%d%%'
                        ORDER BY nid
                        LIMIT 10", $search);
  }
  else {
    // Assume looking for filename.
    $result = db_query("SELECT filename
                        FROM {pifr_file}
                        WHERE filename LIKE '%s%%'
                        ORDER BY filename
                        LIMIT 10", $search);
  }

  // Populate suggestions.
  $suggestions = array();
  while ($suggestion = db_result($result)) {
    $suggestions[$suggestion] = $suggestion;
  }

  drupal_json($suggestions);
}

/**
 * View the files under review for a particular node from a project server.
 *
 * @param interger $server_id Server ID or URL that node is found on.
 * @param interger $nid Node ID.
 * @return string HTML output.
 */
function pifr_view_node($server, $nid) {
  // Check if valid server.
  $server = pifr_server_check($server, PIFR_SERVER_PROJECT);

  if ($server) {
    // TODO Add pager.
    $header = array(t('Name'), t('Status'), t('Result'), t('Operations'));
    $rows = array();

    $files = pifr_file_get_all($server->server_id, $nid);
    while ($file = db_fetch_object($files)) {
      $row = array();
      $row[] = $file->filename;
      $row[] = pifr_file_get_status($file);
      $row[] = pifr_file_get_result($file);
      $row[] = ($file->status >= PIFR_FILE_ERROR ? l(t('View results'), 'pifr/file/' . $server->server_id . '/' . $file->file_id) : '');

      $rows[] = $row;
    }

    if (count($rows) == 0) {
      return '<p>' . t('No files to display.') . '</p>';
    }

    drupal_set_title(t('File review for node #@nid', array('@nid' => $nid)));
    drupal_add_feed(url('pifr/rss/node/' . $server->server_id . '/' . $nid, array('absolute' => TRUE)));
    return theme('table', $header, $rows);
  }
  else {
    drupal_set_message(t('Invalid server'), 'error');
    return '';
  }
}

/**
 * Display the test results for a file.
 *
 * @param interger $server_id Server ID or URL.
 * @param integer $file_id File ID or name.
 * @return string HTML output.
 */
function pifr_view_file($server, $file) {
  $server = pifr_server_check($server, PIFR_SERVER_PROJECT);
  if (!$server) {
    drupal_set_message(t('Invalid server'), 'error');
    return '';
  }

  $file = pifr_file_check($file, $server->server_id);
  if ($file) {
    drupal_set_title(t('Review results for %filename', array('%filename' => $file->filename)));
    $output = '';

    // Get general summary information.
    $header = array('', array('data' => t('Local'), 'style' => 'width: 40%;'), array('data' => t('Project'), 'style' => 'width: 40%;'));
    $rows = array();
    $rows[] = array(t('Project'), '', l($server->url, $server->url));
    $rows[] = array(t('Node ID'), l($file->nid, 'pifr/node/' . $server->server_id . '/' . $file->nid), l($file->nid, $server->url . 'node/' . $file->nid));
    $rows[] = array(t('Comment ID'), ($file->cid > 0 ? $file->cid : t('None')),
                                     ($file->cid > 0 ? l($file->cid, $server->url . 'node/' . $file->nid . '#comment-' . $file->cid) : t('None')));
    $rows[] = array(t('File'), '', l($file->filename, pifr_file_get_url($file)));
    $rows[] = array(t('Status'), pifr_file_get_status($file));
    $rows[] = array(t('Received'), format_date($file->received, 'large'));
    $rows[] = array(t('Last tested'), ($file->last_tested ? format_date($file->last_tested, 'large') : '<em>' . t('Never') . '</em>'));
    $rows[] = array(t('Test count'), $file->test_count);
    $rows[] = array(t('Result'), pifr_file_get_result($file));

    // Bold first column.
    foreach ($rows as $i => $row) {
      $rows[$i][0] = '<b>' . $row[0] . '</b>';
      if ($i > 3) {
        $rows[$i][1] = array('data' => $rows[$i][1], 'colspan' => '2');
      }
    }

    // Color the result row.
    if ($file->status >= PIFR_FILE_ERROR) {
      if (($file->status == PIFR_FILE_ERROR || $file->status == PIFR_FILE_TEST_FAIL) && $file->fail + $file->exception == 0) {
        $file->fail = 1; // Trick into applying fail class.
      }
      $rows[8] = array('data' => $rows[8], 'class' => pifr_get_result_class($file));
    }

    $output .= theme('table', $header, $rows);

    // Ouput event log.
    $output .= '<h2>' . t('Event log') . '</h2>';
    $output .= pifr_log_view($file->file_id);

    // Output test results.
    $output .= '<h2>' . t('Test results') . '</h2>';
    $rows = array();

    if ($file->status == PIFR_FILE_TEST_FAIL || $file->status == PIFR_FILE_TEST_PASS) {
      $result = db_query('SELECT test_class, pass, fail, exception
                          FROM {pifr_result}
                          WHERE file_id = %d
                          ORDER BY test_class', $file->file_id);

      $header = array(t('Test class'), t('Pass'), t('Fail'), t('Exception'));
      $rows = array();

      while ($summary = db_fetch_object($result)) {
        $row = array();
        $row[] = $summary->test_class;
        $row[] = $summary->pass;
        $row[] = $summary->fail;
        $row[] = $summary->exception;

        $rows[] = array('data' => $row, 'class' => pifr_get_result_class($summary));
      }
    }

    if (count($rows) == 0) {
      $output .= '<p><em>' . t('No test results to display.') . '</em></p>';
    }
    else {
      $output .= theme('table', $header, $rows);
    }

    drupal_add_css(drupal_get_path('module', 'pifr') . '/pifr.css');
    drupal_add_feed(url('pifr/rss/file/' . $server->server_id . '/' . $file->filename, array('absolute' => TRUE)));
    return $output;
  }
  else {
    drupal_set_message(t('Invalid file.'), 'error');
    return '';
  }
}

/**
 * Get the CSS class appropriate for the result summary.
 *
 * @param object $summary Summary of result, incuding: pass, fail, exception properties.
 * @return string Result class.
 */
function pifr_get_result_class($summary) {
  if ($summary->fail) {
    return 'pifr-fail';
  }
  else if ($summary->exception) {
    return 'pifr-exception';
  }
  return 'pifr-pass';
}

/**
 * Get the status of a file.
 *
 * @param object $file File object.
 * @return unknown
 */
function pifr_file_get_status($file) {
  if ($file->status == PIFR_FILE_QUEUED) {
    return t('Queued');
  }
  else if ($file->status == PIFR_FILE_SENT) {
    return t('Sent to slave @id at @sent', array('@id' => $file->slave_id, '@sent' => format_date($file->sent, 'short')));
  }
  return t('Reviewed');
}

/**
 * Get the the result of a file review.
 *
 * @param object $file File object.
 * @return string Review result.
 */
function pifr_file_get_result($file) {
  if ($file->status <= PIFR_FILE_SENT) {
    return '<em>' . t('Pending') . '</em>';
  }
  // Testing slave completed testing and returned a status code.
  else if ($file->status <= PIFR_FILE_ERROR) {
    // Error while testing file
    return t('Error: @message', array('@message' => $file->message));
  }
  else if ($file->pass + $file->fail + $file->exception == 0) {
    // File failed before running testing suite.
    return t('Failed: @message', array('@message' => $file->message));
  }
  // File made it through testing without an error so test suite was run.
  $message = ($file->status == PIFR_FILE_TEST_PASS ? t('Passed') : t('Failed'));
  $test_summary = _simpletest_format_summary_line(array('#pass' => $file->pass,
                                                        '#fail' => $file->fail,
                                                        '#exception' => $file->exception));
  return t('@message: @test_summary', array('@message' => $message, '@test_summary' => $test_summary));
}


/**
 * View the status of testing.
 *
 * @return string HTML output.
 */
function pifr_view_status() {
  $output = '';

  // Get number of files sent vs queued.
  $queued = db_result(db_query('SELECT COUNT(file_id)
                                FROM {pifr_file}
                                WHERE status = %d', PIFR_FILE_QUEUED));
  $sent = db_result(db_query('SELECT COUNT(file_id)
                              FROM {pifr_file}
                              WHERE status = %d', PIFR_FILE_SENT));
  $total = $queued + $sent;

  if ($total > 0) {
    $chart = array(
      '#chart_id' => 'result_breakdown',
      '#title' => t('Queue'),
      '#type' => CHART_TYPE_PIE_3D,
      '#size' => chart_size(350, 150)
    );

    $chart['#data'] = array($sent, $queued);
    $chart['#labels'] = array(t('Testing: @count', array('@count' => $sent)), t('Queued: @count', array('@count' => $queued)));
    $output .= chart_render($chart);
  }

  // Capacity status.
  $slaves = pifr_server_get_type(PIFR_SERVER_SLAVE);
  $free = pifr_server_free_slaves();

  $percent_used = (count($slaves) - count($free)) / count($slaves) * 100;
  $output .= '<img src="http://chart.apis.google.com/chart?cht=gom&chtt=Capacity&chd=t:' . $percent_used . '&chs=300x150&chf=bg,s,FFFFFF00" />';

  // Individual slave status.
  $header = array(t('Slave ID'), t('Status'));
  if (($manage = user_access('manage pifr'))) {
    $header[] = t('Operations');
  }

  $rows = array();
  foreach ($slaves as $slave) {
    $row = array();
    $row[] = $slave->server_id;
    $row[] = (in_array($slave, $free) ? t('Free') : t('Busy'));
    if ($manage) {
      $row[] = l(t('Reset'), 'pifr/reset/' . $slave->server_id);
    }

    $rows[] = $row;
  }
  $output .= '<h3>' . t('Slaves') . '</h3>';
  $output .= theme('table', $header, $rows);

  return $output;
}

/**
 * View the status of testing.
 *
 * @return string HTML output.
 */
function pifr_view_slave_status() {
  $output = '';
  $num_tests = pifr_count_completed_tests();
  // Note: $last_test_count is the total number of tests run when
  // testing the *last* patch. It is possible that in the current test
  // run there may be a different number of tests.
  $last_test_count = variable_get('pifr_last_test_count', NULL);

  module_load_include('inc', 'pifr', 'pifr.slave');
  if (pifr_slave_test_processes_get()) {
    if (!empty($last_test_count) && !empty($num_tests)) {
      $output .= t('Completed !num_tests of about !total_num_tests tests.', array('!num_tests' => $num_tests, '!total_num_tests' => $last_test_count));
      $output .= theme('pifr_progress_bar', $num_tests, $last_test_count);
    }
    else {
      $output .= t('Completed !num_tests tests.', array('!num_tests' => $num_tests));
    }
  }
  else {
    $output .= t('No tests are running at this time.');
  }

  return $output;
}

/**
 * Get the number of tests that have been completed.
 *
 * @return
 *   The number of tests that have been completed on this testing
 *   run, or null if tests are not currently being run.
 */
function pifr_count_completed_tests() {
  global $db_url;

  $original_db_url = $db_url;

  // Parse the current $db_url into its pieces and reconstruct $db_url
  // to use the database used for the Simpletest tests.
  $url = parse_url(is_array($db_url) ? $db_url['default'] : $db_url);
  $db_user = urldecode($url['user']);
  $db_pass = isset($url['pass']) ? urldecode($url['pass']) : NULL;
  $db_host = urldecode($url['host']);
  $db_port = isset($url['port']) ? urldecode($url['port']) : '';
  $db_path = ltrim(urldecode($url['path']), '/');

  // Build new $db_url to use.
  $db_type = substr($db_url, 0, strpos($db_url, '://'));
  $checkout_db_url = $db_type .'://'. urlencode($db_user) . ($db_pass ? ':'. urlencode($db_pass) : '') .'@'. ($db_host ? urlencode($db_host) : 'localhost') . ($db_port ? ":$db_port" : '') .'/'. urlencode($db_path) . '_checkout';

  if (is_array($db_url)) {
    $db_url['pifr_checkout'] = $checkout_db_url;
  }
  else {
    $db_url = array(
      'default' => $db_url,
      'pifr_checkout' => $checkout_db_url,
    );
  }
  // Use the new database connection.
  db_set_active('pifr_checkout');

  // Count number of completed tests.
  $num_tests = NULL;
  $num_tests = @db_result(db_query('SELECT COUNT(*) FROM {simpletest}'));

  // Restore previous value of $db_url;
  $db_url = $original_db_url;
  db_set_active();
  return $num_tests;
}

/**
 * Reset slave confirmation form.
 */
function pifr_reset_slave_form(&$form_state, $slave_id) {
  $form = array();
  $slave = pifr_server_get($slave_id);

  if (!$slave) {
    drupal_set_message(t('Invalid server'), 'error');
    $form_state['redirect'] = 'pifr/status';
  }
  else {
    $form['slave_id'] = array(
      '#type' => 'value',
      '#value' => $slave->server_id
    );
  }

  return confirm_form($form, t('Reset slave %slave_id?', array('%slave_id' => $slave_id)), 'pifr/status',
                      t('Reset the slave server and any files in the queue related to it.'));
}

/**
 * Reset slave server.
 */
function pifr_reset_slave_form_submit($form, &$form_state) {
  $slave = pifr_server_get($form_state['values']['slave_id']);
  $self = pifr_server_get_type(PIFR_SERVER_TEST_MASTER);

  if ($slave) {
    $new_status = xmlrpc($slave->url . 'xmlrpc.php', 'pifr.reset', $self->server_key);
    if ($new_status == PIFR_STATUS_FREE) {
      drupal_set_message(t('Slave @slave_id reset successfully.', array('@slave_id' => $slave->server_id)));
    }
    else {
      drupal_set_message(t('Failed to reset slave @slave_id.', array('@slave_id' => $slave->server_id)), 'error');
      watchdog('pifr', 'Failed to reset slave @slave_id.', array('@slave_id' => $slave->server_id), WATCHDOG_ERROR);
    }

    // Reset related queue files.
    $files = pifr_file_get_all_sent($slave->server_id);
    foreach ($files as $file_id) {
      pifr_file_set_queued($file_id);
    }
  }
  $form_state['redirect'] = 'pifr/status';
}

/**
 * View stats from testing.
 *
 * @return string HTML output.
 */
function pifr_view_stats() {
  $output = '';

  $results = db_query('SELECT COUNT(file_id) AS count, status, message
                      FROM {pifr_file}
                      WHERE status >= %d
                      GROUP BY status, message
                      ORDER BY count', PIFR_FILE_ERROR);

  $chart = array(
    '#chart_id' => 'result_breakdown',
    '#title' => t('Testing result breakdown'),
    '#type' => CHART_TYPE_PIE_3D,
    '#size' => chart_size(500, 200)
  );

  $chart['#data'] = array();
  $chart['#labels'] = array();
  while ($result = db_fetch_object($results)) {
    $chart['#data'][] = $result->count;

    $message = $result->message;
    if (empty($message)) {
      $message = ($result->status == PIFR_FILE_TEST_PASS ? t('Passed tests.') : t('Failed tests.'));
    }
    $chart['#labels'][] = t($message);
  }
  $output .= chart_render($chart);

  return $output;
}

/**
 * Display basic information about the nodes that have been tested for a
 * particular project server.
 *
 * @param mixed $server Server ID or URL.
 */
function pifr_view_rss($server) {
  $server = pifr_server_check($server, PIFR_SERVER_PROJECT);
  if ($server) {
    $start = ($_REQUEST['start'] ? $_REQUEST['start'] : 0);
    $count = ($_REQUEST['count'] ? $_REQUEST['count'] : 20);
    $results = db_query('SELECT nid, COUNT(*) AS file_count, SUM(test_count) AS test_count
                        FROM {pifr_file}
                        WHERE server_id = %d
                        GROUP BY nid
                        ORDER BY MAX(last_tested) DESC
                        LIMIT %d, %d', $server->server_id, $start, $count);

    $items = '';
    while ($result = db_fetch_array($results)) {
      $result['issue'] = $server->url . 'node/' . $result['nid'];

      $link = url('pifr/node/' . $server->server_id . '/' . $result['nid'], array('absolute' => TRUE));
      $items .= format_rss_item(t('Node #@nid', array('@nid' => $result['nid'])), $link, '', $result);
    }
    echo pifr_output_rss($items, t('Test results for @server.', array('@server' => $server->url)));
  }
  else {
    echo pifr_response_code(PIFR_RESPONSE_INVALID_SERVER);
  }
}

/**
 * Display the summary results for each file int he specified node.
 *
 * @param mixed $server Server ID or URL.
 * @param interger $nid Node ID.
 */
function pifr_view_rss_node($server, $nid) {
  $server = pifr_server_check($server, PIFR_SERVER_PROJECT);
  if ($server) {
    if (is_numeric($nid)) {
      $results = db_query('SELECT cid, filename, status, received, sent, last_tested, test_count, message, pass, exception, fail
                           FROM {pifr_file}
                           WHERE server_id = %d
                           AND nid = %d', $server->server_id, $nid);
      $items = '';
      while ($result = db_fetch_array($results)) {
        $link = url('pifr/file/' . $server->server_id . '/' . $result['filename'], array('absolute' => TRUE));
        $file = pifr_file_find($result['filename'], $server->server_id);
        $items .= format_rss_item($result['filename'], $link, pifr_file_get_result($file), $result);
      }
      echo pifr_output_rss($items, t('Test results for node #@nid.', array('@nid' => $nid)));
    }
    else {
      echo 'Invalid node ID';
    }
  }
  else {
    echo pifr_response_code(PIFR_RESPONSE_INVALID_SERVER);
  }
}

/**
 * Display the testing results for the specified file.
 *
 * @param mixed $server Server ID or URL.
 * @param mixed $filename Filename or file ID.
 */
function pifr_view_rss_file($server, $file) {
  $server = pifr_server_check($server, PIFR_SERVER_PROJECT);
  if ($server) {
    $file = pifr_file_check($file, $server->server_id);
    if ($file) {
      $results = db_query('SELECT test_class, pass, fail, exception
                           FROM {pifr_result}
                           WHERE file_id = %d', $file->file_id);
      $items = '';
      while ($result = db_fetch_array($results)) {
        $test_summary = _simpletest_format_summary_line(array('#pass' => $result['pass'],
                                                              '#fail' => $result['fail'],
                                                              '#exception' => $result['exception']));
        $items .= format_rss_item($result['test_class'], '', $test_summary, $result);
      }
      echo pifr_output_rss($items, t('Test results for @file.', array('@file' => $file->filename)));
    }
    else {
      echo 'Invalid file';
    }
  }
  else {
    echo pifr_response_code(PIFR_RESPONSE_INVALID_SERVER);
  }
}

function pifr_output_rss($items, $description) {
  global $base_url, $language;

  $channel = array(
    'version'     => '2.0',
    'title'       => variable_get('site_name', 'Drupal') . ' - Test results',
    'link'        => $base_url,
    'description' => $description,
    'language'    => $language->language
  );

  $output = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
  $output .= "<rss version=\"". $channel["version"] ."\" xml:base=\"". $base_url ."\">\n";
  $output .= format_rss_channel($channel['title'], $channel['link'], $channel['description'], $items, $channel['language']);
  $output .= "</rss>\n";

  drupal_set_header('Content-Type: application/rss+xml; charset=utf-8');
  return $output;
}

/**
 * PIFR settings form.
 */
function pifr_settings_form(&$form_state) {
  $form = array();

  $form['server_role'] = array(
    '#type' => 'fieldset',
    '#title' => t('Server role(s)'),
    '#description' => t('Please select the role(s) that will be fullfilled by this server.')
  );
  $role = pifr_get_server_role();
  $form['server_role']['pifr_role'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Role(s)'),
    '#default_value' => $role,
    '#options' => array(
      PIFR_SERVER_TEST_MASTER,
      PIFR_SERVER_SLAVE
    )
  );
  $form['server_role']['pifr_role']['#options'] = drupal_map_assoc($form['server_role']['pifr_role']['#options'], 'pifr_server_role');

  if ($role[PIFR_SERVER_TEST_MASTER]) {
    $form['test_master'] = array(
      '#type' => 'fieldset',
      '#title' => t('Testing master'),
      '#description' => t('Please configure the testing master server.')
    );
    $form['test_master']['pifr_test_master_active'] = array(
      '#type' => 'checkbox',
      '#title' => t('Active'),
      '#description' => t('Actively send patches to slaves, uncheck to turn off.'),
      '#size' => 4,
      '#default_value' => variable_get('pifr_test_master_active', FALSE)
    );
    $form['test_master']['pifr_test_timeout'] = array(
      '#type' => 'textfield',
      '#title' => t('Testing timeout'),
      '#description' => t('The maximum time, in minutes, allowed to testing slaves until they are automatically reset.'),
      '#size' => 4,
      '#default_value' => variable_get('pifr_test_timeout', 45)
    );
    $form['test_master']['pifr_test_master_cron_delay'] = array(
      '#type' => 'textfield',
      '#title' => t('Cron delay'),
      '#description' => t('The time to wait, in minutes, before executing the cron code.'),
      '#size' => 4,
      '#default_value' => variable_get('pifr_test_master_cron_delay', 5)
    );
    $form['test_master']['pifr_test_master_report'] = array(
      '#type' => 'checkbox',
      '#title' => t('Report'),
      '#description' => t('Report test results back to project server.'),
      '#default_value' => variable_get('pifr_test_master_report', FALSE)
    );
    $form['test_master']['pifr_test_master_check_do'] = array(
      '#type' => 'checkbox',
      '#title' => t('Check drupal.org'),
      '#description' => t('Check drupal.org for commit to test a clean HEAD against and check for drupal.org availability. ' .
                          'This will enable both "Active" and "Report" if the two conditions are met, if you need to disable ' .
                          'the test server make sure to turn of this option or the server may re-enable itself.'),
      '#default_value' => variable_get('pifr_test_master_check_do', FALSE)
    );
  }

  if ($role[PIFR_SERVER_SLAVE]) {
    $form['test_slave'] = array(
      '#type' => 'fieldset',
      '#title' => t('Testing slave'),
      '#description' => t('Please configure the testing slave.')
    );
    $form['test_slave']['pifr_php_executable'] = array(
      '#type' => 'textfield',
      '#title' => t('PHP Executable'),
      '#description' => t('The location of the PHP executable to run tests with.'),
      '#default_value' => variable_get('pifr_php_executable', '/usr/bin/php')
    );
    $form['test_slave']['pifr_debug'] = array(
      '#type' => 'checkbox',
      '#title' => t('Debug'),
      '#description' => t('Control the output of detailed debug information to the watchdog log.'),
      '#default_value' => variable_get('pifr_debug', FALSE)
    );
  }

  $form = system_settings_form($form);
  $form['#submit'][] = 'pifr_settings_form_submit';

  return $form;
}

/**
 * Update menu items to reflect server role.
 */
function pifr_settings_form_submit($form, &$form_state) {
  menu_rebuild();

  if ($form_state['values']['pifr_role'][PIFR_SERVER_SLAVE]) {
    $checkout = file_directory_path() . '/checkout';
    $review = file_directory_path() . '/review';
    file_check_directory($checkout, TRUE);
    file_check_directory($review, TRUE);

    // TODO Deal with symbolic link.
  }
  else {
    // TODO Properly remove checkout.

    if (file_exists(file_directory_path() . '/review')) {
      rmdir(file_directory_path() . '/review');
    }
  }
}

function theme_pifr_progress_bar($numerator, $denominator) {
    $output = '';

  // Number of characters that will represent 10%.
  $unit_width = 2;
  $completed = $numerator / $denominator;
  if (!empty($completed)) {
    $completed_width = min(round($completed * $unit_width * 10), $unit_width * 10);
    $progress = str_repeat('*', $completed_width) . str_repeat(' ', ($unit_width * 10) - $completed_width);

    $output = '[' . $progress . ']';
  }
  return '<p><pre>' . $output . '</pre></p>';
}
