<?php
// $Id: pifr.server.inc,v 1.9 2008/12/02 06:45:23 boombatower Exp $
/**
 * @file
 * Automatically review patches attached to issues.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Get server by URL.
 *
 * @param string $url Server URL.
 * @param integer $type Server type ID.
 * @return mixed Server object or FALSE.
 */
function pifr_server_find($url, $type) {
  $result = db_query("SELECT server_id, server_key, url, path, type
                      FROM {pifr_server}
                      WHERE url LIKE '%%%s%%'
                      AND type = %d", $url, $type);
  return db_fetch_object($result);
}

/**
 * Get server by server ID.
 *
 * @param integer $server_id Server ID.
 * @return mixed Server object or FALSE.
 */
function pifr_server_get($server_id) {
  $result = db_query('SELECT server_id, server_key, url, path, type
                      FROM {pifr_server}
                      WHERE server_id = %d', $server_id);
  return db_fetch_object($result);
}

/**
 * Get server by server key.
 *
 * @param string $server_key Server key.
 * @return mixed Server object or FALSE.
 */
function pifr_server_get_key($server_key) {
  $result = db_query("SELECT server_id, server_key, url, path, type
                      FROM {pifr_server}
                      WHERE server_key = '%s'", $server_key);
  return db_fetch_object($result);
}

/**
 * Get server by ID or URL.
 *
 * @param mixed $server Server ID or URL.
 * @param integer $type Server type ID.
 * @return mixed Server object or FALSE.
 */
function pifr_server_check($server, $type) {
  if (is_numeric($server)) {
    $server = pifr_server_get($server);
    return ($server && $server->type == $type ? $server : FALSE);
  }
  return pifr_server_find($server, $type);
}

/**
 * Get server by key.
 *
 * @param string $server_key Server key.
 * @param integer $type Server type ID.
 * @return mixed Server object or FALSE.
 */
function pifr_server_check_key($server_key, $type) {
  $server = pifr_server_get_key($server_key);
  if ($server && $server->type == $type) {
    return $server;
  }
  return FALSE;
}

/**
 * Get all server objects.
 *
 * @return array List of server objects.
 */
function pifr_server_get_all() {
  $result = db_query('SELECT server_id, server_key, url, path, type
                      FROM {pifr_server}');

  $servers = array();
  while ($server = db_fetch_object($result)) {
    $servers[] = $server;
  }
  return $servers;
}


/**
 * Get server by type. If type is Test master then it will only return the
 * first one as there should only be one test master.
 *
 * @param interger $server_type Server type.
 * @return mixed Array of server or single Test master.
 */
function pifr_server_get_type($server_type) {
  $result = db_query('SELECT server_id, server_key, url, path, type
                      FROM {pifr_server}
                      WHERE type = %d', $server_type);

  if ($server_type == PIFR_SERVER_TEST_MASTER) {
    return db_fetch_object($result);
  }

  $servers = array();
  while ($server = db_fetch_object($result)) {
    $servers[] = $server;
  }
  return $servers;
}

/**
 * Attempt to find a free slaves.
 *
 * @return array List of slave objects.
 */
function pifr_server_free_slaves() {
  $slaves = pifr_server_get_type(PIFR_SERVER_SLAVE);
  $free = array();
  foreach ($slaves as $slave) {
    // Ping each slave until an available one is found.
    $result = xmlrpc($slave->url . 'xmlrpc.php', 'pifr.status');
    if ($result !== FALSE && $result == PIFR_STATUS_FREE) {
      // Server is not reviewing and is thus free.
      $free[] = $slave;
    }
  }
  return $free;
}

/**
 * Check all slave server statuses and confirm that the results are the
 * expected results. If not then update the slave server and file queue to
 * reflect the current status.
 */
function pifr_server_check_slaves() {
  $slaves = pifr_server_get_type(PIFR_SERVER_SLAVE);
  $free_slaves = pifr_server_free_slaves();
  $test_timeout = variable_get('pifr_test_timeout', 45);
  $self = pifr_server_get_type(PIFR_SERVER_TEST_MASTER);
  foreach ($slaves as $slave) {
    // Get the files in the queue related to the slave.
    $files = pifr_file_get_all_sent($slave->server_id);

    if (in_array($slave, $free_slaves)) {
      // If slave is free and there is a file in queue marked as have been sent
      // to that server then reset queue status.
      foreach ($files as $file_id) {
        pifr_file_set_queued($file_id);
      }
    }
    else {
      // If slave is busy, but has been so for test_timeout then reset server and
      // file in queue.
      foreach ($files as $file_id) {
        $file = pifr_file_get($file_id);
        if ($file->sent + ($test_timeout * 60) <= time()) {
          // File sent longer than test_timeout ago so reset slave and file.
          $new_status = xmlrpc($slave->url . 'xmlrpc.php', $self->server_key, 'pifr.reset');
          if ($new_status != PIFR_STATUS_FREE) {
            watchdog('pifr', t('Failed to reset slave @slave_id.', array('@slave_id' => $slave->server_id)), WATCHDOG_ERROR);
          }
          pifr_file_set_queued($file->file_id);
        }
      }
    }
  }
}
