<?php
// $Id: pifr.slave.inc,v 1.4 2008/12/12 04:14:45 boombatower Exp $
/**
 * @file
 * Automatically review patches attached to issues.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

// This needs to be the same as the DRUPAL_MINIMUM_MYSQL in Drupal HEAD.
define('PIFR_MINIMUM_MYSQL', '5.0');

/**
 * Test the server configuration and display the results.
 *
 * @return string HTML output.
 */
function pifr_slave_check_configuration() {
  drupal_set_title(t('Slave configuration test'));
  drupal_add_css(drupal_get_path('module', 'pifr') . '/pifr.css');
  module_load_include('inc', 'pifr', 'pifr.review');
  $output = '';

  $output .= '<h3>' . t('Firewall/connection') . '</h3>';
  $output .= '<div class="pifr-test-section">';
  // Test the firewall rules to make sure that the server can
  // communicate with itself (localhost), drupal.org, testing.drupal.org,
  // and cvs.drupal.org as required.
  $connections = array(
    'required' => array(
      'title' => t('Required'),
      'description' => t('All of the following connection tests must pass in order for your slave to be able to properly communicate with the testing server.'),
      'tests' => array(
        array(
          'hostname' => 'drupal.org',
          'port' => 80,
        ),
        array(
          'hostname' => 'testing.drupal.org',
          'port' => 80,
        ),
        array(
          'hostname' => '88.208.239.81',
          'port' => 80,
        ),
        array(
          'hostname' => 'cvs.drupal.org',
          'port' => 2401,
        ),
        array(
          'hostname' => 'localhost',
          'port' => 80,
        ),
      ),
    ),
    'recommended' => array(
      'title' => t('Recommended'),
      'description' => t('Because of the possibility that insecure code in a patch being tested on this machine could be executed, it is recommended that outgoing connections from this server be blocked except when required.  <em>Therefore, for the most secure configuration, all of the following tests should fail.</em>'),
      'tests' => array(
        array(
          'hostname' => 'www.google.com',
          'port' => 80,
        ),
        array(
          'hostname' => 'mail.google.com',
          'port' => 25,
        ),
      ),
    ),
  );

  foreach ($connections as $key => $connection) {
    $output .= '<h4 class=\"pifr-test-connection-title\">' . $connection['title'] . '</h4>';
    $output .= '<div class=\"pifr-test-connection-description\">' . $connection['description'] . '</div>';
    $header = array(t('Host name'), t('Port'), t('Result'), t('Error message'));
    $rows = array();
    foreach ($connection['tests'] as $test) {
      $result = pifr_slave_test_connection($test['hostname'], $test['port']);
      if (!empty($result['error_number'])) {
        $rows[] = array($test['hostname'], $test['port'], t('Failed'), $result['error_message']);
      }
      else {
        $rows[] = array($test['hostname'], $test['port'], t('Passed'), '');
      }
    }
    $output .= theme('table', $header, $rows);
  }
  $output .= '</div>';

  // Check for required command line tools.
  $output .= '<h3>' . t('Command line tools') . '</h3>';
  $output .= '<div class="pifr-test-section">';
  $header = array(t('Name'), t('Shell command'), t('Output'));
  $rows = array();

  // Check for patch.
  $cmd = 'patch -v';
  $cmd_output = shell_exec($cmd);
  $rows[] = array('patch', $cmd, '<code>' . check_plain($cmd_output) . '</code>');

  // Check for gawk.
  $cmd = 'gawk --version';
  $cmd_output = shell_exec($cmd);
  $rows[] = array('gawk', $cmd, '<code>' . check_plain($cmd_output) . '</code>');

  // Check for cvs.
  $cmd = 'cvs -v';
  $cmd_output = shell_exec($cmd);
  $rows[] = array('cvs', $cmd, '<code>' . check_plain($cmd_output) . '</code>');

  // Check for curl.
  $cmd = 'curl --version';
  $cmd_output = shell_exec($cmd);
  $rows[] = array('curl', $cmd, '<code>' . check_plain($cmd_output) . '</code>');

  // Check for php cli.
  $cmd = 'php -v';
  $cmd_output = shell_exec($cmd);
  $rows[] = array('php (cli)', $cmd, '<code>' . check_plain($cmd_output) . '</code>');

  // Check for grep.
  $cmd = 'grep --version';
  $cmd_output = shell_exec($cmd);
  $rows[] = array('grep', $cmd, '<code>' . check_plain($cmd_output) . '</code>');

  $output .= theme('table', $header, $rows);
  $output .= '</div>';

  // Check for required PHP Extensions.
  $output .= '<h3>' . t('PHP Extensions') . '</h3>';
  $output .= '<div class="pifr-test-section">';
  $header = array(t('Extension name'), t('Minimum required version'), t('Version found'), t('Result'));
  $rows = array();

  // Check for CURL.
  $min_version = '7';
  $found_version = '';
  $result = FALSE;
  if (function_exists('curl_version')) {
    $info = curl_version();
    $found_version = $info['version'];
    if (version_compare($min_version, $found_version) <= 0) {
      $result = TRUE;
    }
  }
  $rows[] = array(l('CURL', 'http://www.php.net/manual/en/book.curl.php', array('absolute' => TRUE)), $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  // Check for presence of gd library.
  $min_version = '2';
  $found_version = '2';
  $result = FALSE;
  if (function_exists('imagegd2')) {
    $result = TRUE;
  }
  $rows[] = array(l('gd', 'http://www.php.net/manual/en/ref.image.php', array('absolute' => TRUE)), $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  // Check for presence of mbstring library.
  $min_version = UNICODE_MULTIBYTE;
  $found_version = -1;
  $result = FALSE;
  $found_version = $GLOBALS['multibyte'];
  if ($min_version <= $found_version) {
    $result = TRUE;
  }
  $rows[] = array(l('mbstring', 'http://www.php.net/manual/en/book.mbstring.php', array('absolute' => TRUE)), $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  // Check for correct MySQL version.
  $min_version = PIFR_MINIMUM_MYSQL;
  $found_version = 0;
  $result = FALSE;
  $found_version = db_version();
  if ($min_version <= $found_version) {
    $result = TRUE;
  }
  $rows[] = array(l('mysql', 'http://www.php.net/manual/en/book.mysql.php', array('absolute' => TRUE)), $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  // Check for PDO.
  $min_version = 1;
  $found_version = 0;
  $result = FALSE;
  $found_version = phpversion('pdo');
  if ($min_version <= $found_version) {
    $result = TRUE;
  }
  $rows[] = array(l('pdo', 'http://www.php.net/manual/en/book.pdo.php', array('absolute' => TRUE)), $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  $output .= theme('table', $header, $rows);
  $output .= '</div>';
  return $output;
}

/**
 * Test connectivity to a host via a certain port.
 *
 * @param string $hostname A host name or IP address to connect to.
 * @param integer $port The port number to connect to.
 * @param interger $timeout The number of seconds to wait before timing out on
 *   the connection attempt.
 * @return
 *   A keyed array containing the error number and error string as returned
 *   by the PHP function fsockopen().
 */
function pifr_slave_test_connection($hostname, $port, $timeout = 2) {
  $error_number = NULL;
  $error_string = '';
  $fp = @fsockopen($hostname, $port, $error_number, $error_message, $timeout);
  if ($fp) {
    fclose($fp);
  }
  return array('error_number' => $error_number, 'error_message' => $error_message);
}

/**
 * Find any PHP processes that are running the tests.
 *
 * @return array List of process IDs (pids).
 */
function pifr_slave_test_processes_get() {
  $pids = array();

  exec('ps aux | grep php', $output);
  foreach ($output as $line) {
    if (preg_match('/^.*? (\d+ ).*?php \\.\\/scripts\\/run-tests\\.sh/m', $line, $match)) {
      $pids[] = $match[1];
    }
  }

  return $pids;
}

/**
 * Kill the specified list of processes.
 *
 * @param array $pids List of process IDs (pids).
 */
function pifr_slave_test_processes_kill($pids) {
  foreach ($pids as $pid) {
    exec('kill ' . $pid);
  }
}
